@@ -1029,6 +1029,7 @@ _fr_command_add (FrCommand      *self,
 
 		for (scan = new_file_list; scan != NULL; scan = scan->next) {
 			char *filename = scan->data;
+
 			if (find_file_in_archive (archive, filename))
 				del_list = g_list_prepend (del_list, filename);
 		}
@@ -1108,6 +1109,7 @@ _fr_command_add (FrCommand      *self,
 		if (use_tmp_subdirectory) {
 			fr_process_begin_command (self->process, "mv");
 			fr_process_add_arg (self->process, "-f");
+			fr_process_add_arg (self->process, "--");
 			fr_process_add_arg (self->process, tmp_archive_filename);
 			fr_process_add_arg (self->process, archive_filename);
 			fr_process_end_command (self->process);
@@ -1152,10 +1154,14 @@ process_ready_after_changing_archive (GObject      *source_object,
 				      gpointer      user_data)
 {
 	XferData *xfer_data = user_data;
-	FrError  *error = NULL;
+	GError   *error = NULL;
 
-	if (! fr_process_execute_finish (FR_PROCESS (source_object), result, &error)) {
-		g_simple_async_result_set_from_error (xfer_data->result, error->gerror);
+	if (! fr_command_handle_process_error (FR_COMMAND (xfer_data->archive), result, &error))
+		/* command restarted */
+		return;
+
+	if (error != NULL) {
+		g_simple_async_result_set_from_error (xfer_data->result, error);
 	}
 	else {
 		FrArchive *archive = xfer_data->archive;
@@ -1182,7 +1188,7 @@ process_ready_after_changing_archive (GObject      *source_object,
 
 	g_simple_async_result_complete_in_idle (xfer_data->result);
 
-	fr_error_free (error);
+	g_clear_error (&error);
 	xfer_data_free (xfer_data);
 }
 
@@ -1652,6 +1658,7 @@ _fr_command_remove (FrCommand     *self,
 
 	fr_process_begin_command (self->process, "mv");
 	fr_process_add_arg (self->process, "-f");
+	fr_process_add_arg (self->process, "--");
 	fr_process_add_arg (self->process, tmp_archive_filename);
 	fr_process_add_arg (self->process, archive_filename);
 	fr_process_end_command (self->process);
@@ -1766,6 +1773,7 @@ move_files_to_dir (FrCommand *self,
 		fr_process_add_arg (self->process, "-f");
 	else
 		fr_process_add_arg (self->process, "-n");
+	fr_process_add_arg (self->process, "--");
 	for (scan = list; scan; scan = scan->next) {
 		char *filename = scan->data;
 
@@ -2443,6 +2451,38 @@ fr_command_test_integrity (FrArchive           *archive,
 /* -- fr_command_rename -- */
 
 
+static void
+fr_command_rename__delete (FrArchive   *archive,
+		   	   GList      **file_list,
+		   	   gboolean     is_dir,
+		   	   gboolean     dir_in_archive,
+		   	   const char  *original_path)
+{
+	FrCommand *self = FR_COMMAND (archive);
+	gboolean   added_dir;
+
+	/* temporarily add the dir to rename to the list if it's stored in the
+	 * archive, this way it will be removed from the archive... */
+	added_dir = FALSE;
+	if (is_dir && dir_in_archive && ! g_list_find_custom (*file_list, original_path, (GCompareFunc) strcmp)) {
+		*file_list = g_list_prepend (*file_list, g_strdup (original_path));
+		added_dir = TRUE;
+	}
+
+	_fr_command_remove (self, *file_list, archive->compression);
+
+	/* ...and remove it from the list again */
+	if (added_dir) {
+		GList *tmp;
+
+		tmp = *file_list;
+		*file_list = g_list_remove_link (*file_list, tmp);
+
+		_g_string_list_free (tmp);
+	}
+}
+
+
 static void
 fr_command_rename (FrArchive           *archive,
 		   GList               *file_list,
@@ -2458,7 +2498,6 @@ fr_command_rename (FrArchive           *archive,
 {
 	FrCommand *self = FR_COMMAND (archive);
 	GFile     *tmp_dir;
-	gboolean   added_dir;
 	char      *new_dirname;
 	GList     *new_file_list;
 	GList     *scan;
@@ -2473,6 +2512,9 @@ fr_command_rename (FrArchive           *archive,
 	tmp_dir = _g_file_get_temp_work_dir (NULL);
 
 	fr_process_clear (self->process);
+
+	/* extract the files to rename */
+
 	_fr_command_extract (self,
 			     file_list,
 			     tmp_dir,
@@ -2482,26 +2524,17 @@ fr_command_rename (FrArchive           *archive,
 			     FALSE,
 			     archive->password);
 
-	/* temporarily add the dir to rename to the list if it's stored in the
-	 * archive, this way it will be removed from the archive... */
-	added_dir = FALSE;
-	if (is_dir && dir_in_archive && ! g_list_find_custom (file_list, original_path, (GCompareFunc) strcmp)) {
-		file_list = g_list_prepend (file_list, g_strdup (original_path));
-		added_dir = TRUE;
-	}
+	/* if the command can delete all the files in the archive without
+	 * deleting the archive itself ('rar' deletes the archive), delete the
+	 * files here, that is before adding the renamed files to the archive,
+	 * to make the operation faster. */
 
-	_fr_command_remove (self, file_list, archive->compression);
-
-	/* ...and remove it from the list again */
-	if (added_dir) {
-		GList *tmp;
-
-		tmp = file_list;
-		file_list = g_list_remove_link (file_list, tmp);
-
-		g_free (tmp->data);
-		g_list_free (tmp);
-	}
+	if (archive->propCanDeleteAllFiles)
+		fr_command_rename__delete (archive,
+					   &file_list,
+					   is_dir,
+					   dir_in_archive,
+					   original_path);
 
 	/* rename the files. */
 
@@ -2516,6 +2549,7 @@ fr_command_rename (FrArchive           *archive,
 
 		fr_process_begin_command (self->process, "mv");
 		fr_process_add_arg (self->process, "-f");
+		fr_process_add_arg (self->process, "--");
 		fr_process_add_arg_file (self->process, old_file);
 		fr_process_add_arg_file (self->process, new_file);
 		fr_process_end_command (self->process);
@@ -2541,6 +2575,7 @@ fr_command_rename (FrArchive           *archive,
 		if (! is_dir) {
 			fr_process_begin_command (self->process, "mv");
 			fr_process_add_arg (self->process, "-f");
+			fr_process_add_arg (self->process, "--");
 			fr_process_add_arg_file (self->process, old_file);
 			fr_process_add_arg_file (self->process, new_file);
 			fr_process_end_command (self->process);
@@ -2594,6 +2629,17 @@ fr_command_rename (FrArchive           *archive,
 	g_free (new_dirname);
 	_g_string_list_free (new_file_list);
 
+	/* if the command cannot delete all the files in the archive without
+	 * deleting the archive itself ('rar' deletes the archive), delete
+	 * the old files here to avoid a potential error. */
+
+	if (! archive->propCanDeleteAllFiles)
+		fr_command_rename__delete (archive,
+					   &file_list,
+					   is_dir,
+					   dir_in_archive,
+					   original_path);
+
 	/* remove the tmp dir */
 
 	fr_process_begin_command (self->process, "rm");
@@ -2658,8 +2704,10 @@ fr_command_paste_clipboard (FrArchive           *archive,
 		    && (old_name[strlen (old_name) - 1] != '/'))
 		{
 			fr_process_begin_command (command->process, "mv");
+			fr_process_set_ignore_error (command->process, TRUE);
 			fr_process_set_working_dir_file (command->process, tmp_dir);
 			fr_process_add_arg (command->process, "-f");
+			fr_process_add_arg (command->process, "--");
 			if (old_name[0] == '/')
 				old_name = old_name + 1;
 			fr_process_add_arg (command->process, old_name);
